#!/usr/bin/env python
import sys

if len(sys.argv) == 1:
    import dragon as system
else:
    system = __import__(sys.argv[1].replace('.py',''))

try:
    import sfml as sf
except ImportError:
    raise ImportError("SFML library not found.\n" + 
                      "    Download SFML from http://www.sfml-dev.org/\n" +
                      "    Download Python bindings from http://tinyurl.com/3z57o6v"
                      )
from math import sqrt

from L_system import *
from L_shape import *

class LApp:
    def __init__(self, start, grow_rules, draw_rules):
        self.window = sf.RenderWindow(
               sf.VideoMode.get_fullscreen_modes()[0],
               'L-System',
               sf.Style.FULLSCREEN,
               sf.ContextSettings(antialiasing = 32))

        self.start = start
        self.grow_rules = grow_rules
        self.draw_rules = draw_rules
        self.iterations = 1
        
        try:
            self.font = sf.Font.load_from_file(
                '/System/Library/Fonts/Helvetica.dfont')
        except sf.PySFMLException:
            self.font = sf.Font.DEFAULT_FONT
        
        self.render()
        self.running = True
    
    def render(self):
        '''Calculate the L-shape and set the view based on shape size.'''
        self.vertices = build_shape(
            build_string(self.start, self.grow_rules, self.iterations),
            self.draw_rules)
            
        self.shape = LShape(self.vertices)
        self.fit_view()
        
        self.shape.fill_color = sf.Color(0, 0, 0, 0)
        self.shape.outline_color = sf.Color(255, 255, 255)
        self.shape.outline_thickness = self.outline
    
    def fit_view(self):
        '''Based on vertex positions, calculates an appropriate view.'''
        self.system_view = self.shape.fit_view(self.window)
        self.outline = sqrt(max(self.system_view.width,
                                self.system_view.height)) * 0.01
        if self.shape.closed:
            self.outline *= 2
    
    def draw(self):
        '''Draws the stored L-shape.'''
        self.window.clear(sf.Color(0, 0, 0))
        self.window.view = self.system_view
        self.window.draw(self.shape)

        self.window.view = self.window.default_view        
        t = sf.Text("%i" % self.iterations,
                    character_size = self.window.height/20.)
        t.font = self.font
        t.move(t.local_bounds.height,
               self.window.height - t.local_bounds.height * 2.0)
        self.window.draw(t)
        
        self.window.display()

    def handle_events(self):
        ''' Iterate through window events.  Stops main loop if the window
            is closed, escape is pressed, or command+Q is pressed.'''
        # Check if the GUI window is still open
        for event in self.window.iter_events():
            if event.type == sf.Event.CLOSED:
                self.running = False
            elif event.type == sf.Event.KEY_PRESSED:
                if event.code == sf.Keyboard.ESCAPE:
                    self.running = False
                elif event.system and event.code == sf.Keyboard.Q:
                    self.running = False
                elif event.code == sf.Keyboard.UP:
                    self.iterations += 1
                    self.render()
                elif event.code == sf.Keyboard.DOWN:
                    self.iterations -= 1
                    if self.iterations < 0:
                        self.iterations = 0
                    self.render()
                     
    def loop(self):
        '''Simple main loop.  Handles events and redraws.'''
        while self.running:
            self.handle_events()
            self.draw()
            
if __name__ == '__main__':
    app = LApp(system.start, system.grow, system.draw)
    app.loop()